@@ -4,7 +4,7 @@ package CracTools::Interval::Query;
 }
 # ABSTRACT: Query intervals in various type of files
 #
-$CracTools::Interval::Query::VERSION = '1.04';
+$CracTools::Interval::Query::VERSION = '1.08';
 use strict;
 use warnings;
 
@@ -103,14 +103,13 @@ sub fetchAllNearestDown {
   my @lines;
 
   my $nearest_down = $self->fetchNearestDown($chr,$position,$strand); 
-
   if(defined $nearest_down) {
     my $intervals = $self->_getIntervals($nearest_down);
     # We try to determinate wich interval was matched
     my $best_interval;
     foreach my $i (@$intervals) {
-      $i->{strand} = 1 unless defined $i->{strand};
-      if ( $i->{high}    <  $position && 
+	$i->{strand} = 1 unless defined $i->{strand};
+	if ( $i->{high}    <  $position && 
            $i->{seqname} eq $chr      && 
            $i->{strand}  eq $strand ) {
         if(!defined $best_interval) {
@@ -199,7 +198,7 @@ sub fetchAllNearestUp {
 sub _getIntervals {
   my ($self,$line) = @_;
   my $intervals = $self->{get_interval_sub}->($line);
-  foreach (@$intervals) {$_->{strand} = 1 if !defined $_{strand};}
+  foreach (@$intervals) {$_->{strand} = 1 if !defined $_->{strand};}
   return $intervals;
 }
 
@@ -372,7 +371,7 @@ CracTools::Interval::Query - Query intervals in various type of files
 
 =head1 VERSION
 
-version 1.04
+version 1.08
 
 =head2 new
 
